from lxml import etree
import re
import uuid
import math
import logging

from .default import DefaultTagProcessor


logger = logging.getLogger(__name__)
processors = {}


def tag_processor(tagname):
    def register_tag_processor(cls):
        processors[tagname] = cls()
        return cls
    return register_tag_processor


@tag_processor('SolidColor')
class SolidColorTagProcessor(DefaultTagProcessor):
    def on_start(self, target):
        color = target.attrs.get('color', '#000000')
        target.element.set(
            'fill', color
        )
        if 'fill-opacity' in target.element.attrib:
            del target.element.attrib['fill-opacity']

    def on_end(self, target):
        pass


def rotate_point(x, y, a):
    alpha = a / 180 * math.pi 
    return (
        x * math.cos(alpha) - y * math.sin(alpha),
        x * math.sin(alpha) + y * math.cos(alpha),
    )


def get_coordinates(element):
    if element.tag == "rect":
        x = float(element.attrib.get('x', '0'))
        y = float(element.attrib.get('y', '0'))
        return x, y
    if element.tag == "path":
        x, y, *_ = re.findall(r'\d+\.\d+', element.attrib['d'])
        return float(x), float(y)
    return 0, 0


@tag_processor('RadialGradient')
class RadialGradientTagProcessor(DefaultTagProcessor):
    def on_start(self, target):
        x = float(target.attrs['x'])
        y = float(target.attrs['y'])
        scaleX = float(target.attrs['scaleX'])
        scaleY = float(target.attrs['scaleY'])
        offsetX, offsetY = get_coordinates(target.element)
        cx = offsetX + x
        cy = offsetY + y
        rx = scaleX / 2
        ry = scaleY / 2
        target.element = etree.SubElement(
            target.element,
            "radialGradient",
            cx=str(cx),
            cy=str(cy),
            rx=str(rx),
            ry=str(ry),
            id=str(uuid.uuid4())
        )

    def on_end(self, target):
        element = target.element
        target.element = element.getparent()
        target.defs.append(element)
        target.element.set('fill', 'url(#%s)' % element.attrib['id'])


@tag_processor('LinearGradient')
class LinearGradientTagProcessor(DefaultTagProcessor):
    def on_start(self, target):
        x = float(target.attrs['x'])
        y = float(target.attrs['y'])
        scaleX = float(target.attrs['scaleX'])
        rotation = float(target.attrs.get('rotation', '0'))
        offsetX, offsetY = get_coordinates(target.element)
        x1 = offsetX + x
        x2 = offsetX + x + scaleX
        y1 = offsetY + y
        y2 = offsetY + y
        x1, y1 = rotate_point(x1, y1, rotation)
        x2, y2 = rotate_point(x2, y2, rotation)
        target.element = etree.SubElement(
            target.element,
            "linearGradient",
            x1=str(x1),
            y1=str(y1),
            x2=str(x2),
            y2=str(y2),
            id=str(uuid.uuid4())
        )

    def on_end(self, target):
        element = target.element
        target.element = element.getparent()
        target.defs.append(element)
        target.element.set('fill', 'url(#%s)' % element.attrib['id'])


@tag_processor('GradientEntry')
class GradientEntryTagProcessor(DefaultTagProcessor):
    def on_start(self, target):
        target.element = etree.SubElement(
            target.element,
            "stop",
            offset=target.attrs.get('ratio', '1'),
            style="stop-color:%s" % target.attrs.get('color', '#000000'),
        )



from lxml import etree
import uuid

from ..utils import process_transform
from .default import DefaultTagProcessor


processors = {}


def tag_processor(tagname):
    def register_tag_processor(cls):
        processors[tagname] = cls()
        return cls
    return register_tag_processor


@tag_processor('DropShadowFilter')
class PathTagProcessor(DefaultTagProcessor):
    def on_start(self, target):
        attrs = target.attrs
        params = {}
        defs = target.defs
        filter_params = {
            'id': str(uuid.uuid4()),
            'width': '250%',
            'height': '250%',
            'x': '-50%',
            'y': '-50%'
        }
        filter = etree.SubElement(
            defs,
            "filter",
            **filter_params
        )
        offset_params = {}
        blur_params = {}
        ct_params = {}

        if 'blurX' in attrs and 'blurY' in attrs:
            if round(float(attrs['blurX'])) == round(float(attrs['blurY'])):
                blur_params['stdDeviation'] = attrs['blurX']
            else:
                blur_params['stdDeviation'] = "%s %s" % (
                    attrs['blurX'], attrs['blurY'])

        if 'distance' in attrs:
            offset_params['dx'] = attrs['distance']
            offset_params['dy'] = attrs['distance']

        current_state = 'SourceAlpha'
        if len(offset_params.keys()) > 0:
            offset_params['in'] = current_state
            offset_params['result'] = 'offsetResult'
            current_state = offset_params['result']

            etree.SubElement(
                filter,
                "feOffset",
                **offset_params
            )

        if len(blur_params.keys()) > 0:
            blur_params['in'] = current_state
            blur_params['result'] = 'blurResult'
            current_state = blur_params['result']
            etree.SubElement(
                filter,
                "feGaussianBlur",
                **blur_params
            )

        if 'alpha' in attrs:
            ct = etree.SubElement(
                filter,
                "feComponentTransfer",
                **{'in': current_state, 'result': 'ctResult'}
            )
            etree.SubElement(
                ct,
                "feFuncA",
                **{"type": "linear", "slope": attrs['alpha']}
            )
            current_state = 'ctResult'

        merge = etree.SubElement(
            filter,
            "feMerge"
        )
        etree.SubElement(
            merge,
            "feMergeNode",
            **{"in": current_state}
        )
        etree.SubElement(
            merge,
            "feMergeNode",
            **{"in": "SourceGraphic"}
        )
                
        target.element.set('filter', 'url(#%s)' % filter_params['id'])
        return target.element

    def on_end(self, target):
        pass



from lxml import etree
import logging


logger = logging.getLogger(__name__)


class DefaultTagProcessor(object):

    def on_start(self, target):
        target.element = etree.SubElement(
            target.element,
            "foreignObject"
        )

    def on_end(self, target):
        parent = target.element.getparent()
        if parent is not None:
            target.element = parent


    def on_data(self, target):
        target.element.text = target.text



from lxml import etree

from .default import DefaultTagProcessor


processors = {}


def tag_processor(tagname):
    def register_tag_processor(cls):
        processors[tagname] = cls()
        return cls
    return register_tag_processor


@tag_processor('SolidColorStroke')
class SolidColorStrokeTagProcessor(DefaultTagProcessor):
    def on_start(self, target):
        color = target.attrs.get('color', '#000000')
        target.element.set(
            'stroke', color
        )
        strokewidth = target.attrs.get('weight')
        if strokewidth:
            target.element.set(
                'stroke-width', strokewidth 
            )
        strokemiterlimit = target.attrs.get('miterLimit')
        if strokemiterlimit:
            target.element.set(
                'stroke-miterlimit', strokemiterlimit 
            )

    def on_end(self, target):
        pass



#!/usr/bin/env python
from lxml import etree
import logging
import sys
import argparse
from . import fxg2svg


LOGLEVELMAP = {
    'debug': logging.DEBUG,
    'info': logging.INFO,
    'warning': logging.WARNING,
    'error': logging.ERROR,

}


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("fxg", help="fxg file")
    parser.add_argument(
        "-l", 
        "--log-level", 
        choices=LOGLEVELMAP.keys(),
        default="info",
        help="set log level"
    )
    parser.add_argument("-o", "--output", help="output svg filename")
    parser.add_argument("-f", "--fontdir", help="directory to fonts")
    args = parser.parse_args()

    fxg_bytes = open(args.fxg, 'r')

    logging.basicConfig(
        format=u'%(levelname)-6s| %(message)s',
        level=LOGLEVELMAP[args.log_level]
    )

    if args.output is not None:
        svg = args.output
    else:
        svg = "%s.svg" % ".".join(args.fxg.split('.')[:-1])
    fxg2svg(fxg_bytes, args.fontdir).getroottree().write(svg, encoding="UTF-8", xml_declaration=True)


if __name__ == "__main__":
    main(sys.argv[1::])



from lxml import etree
from .processors import all_processors as procs
import logging


logger = logging.getLogger(__name__)


def remove_namespace(namespaced):
    *_, unnamespaced = namespaced.split("}")
    return unnamespaced


def process_attrs(attrs):
    new_attrs = {}
    for attrname, attrvalue in attrs.items():
        new_attrs[remove_namespace(attrname)] = attrvalue
    return new_attrs


class SVGTarget(object):

    def __init__(self, fontdir):
        self.element = None
        self.tagname = None
        self.attrs = {}
        self.depth = 0
        self.text = None
        self.svg = []
        self.fxg = []
        self.fontdir = fontdir

    def start(self, tagname, attrs):
        self.tagname = remove_namespace(tagname)
        self.attrs = process_attrs(attrs)
        procs[self.tagname].on_start(self)
        self.svg.append((self.element.tag, self.element.attrib))
        self.fxg.append((self.tagname, self.attrs))

    def end(self, tagname):
        self.tagname = remove_namespace(tagname)
        try:
            self.svg.pop()
            self.fxg.pop()
        except IndexError:
            pass
        procs[self.tagname].on_end(self)

        self.text = None
        self.attrs = {}
        self.tagname = None

    def data(self, data):
        self.text = data
        procs[self.tagname].on_data(self)

    def close(self):
        return self.root

    @property
    def root(self):
        return self.element.getroottree().getroot()

    @property
    def defs(self):
        return self.element.getroottree().find('defs')

    def get_attribute(self, attrname):
        for event in self.fxg:
            tag, attrs = event
            if attrname in attrs:
                return attrs[attrname]



import logging
import os
from PIL import ImageFont


logger = logging.getLogger(__name__)


class FontNotFoundError(Exception):
    pass


def get_filepaths(directory):
    file_paths = []
    for root, directories, files in os.walk(directory):
        for filename in files:
            filepath = os.path.join(root, filename)
            file_paths.append(filepath)  
    return file_paths


def _getfontname(filepath):
    try:
        f = ImageFont.truetype(filepath)
        return f.font.family
    except OSError:
        logger.debug('%s is not a true type font' % filepath)
        return None


def _getfontfile(font_family, f, fontdir):
    for filepath in get_filepaths(fontdir):
        if _getfontname(filepath) == font_family:
            return filepath


def get_font(name, f='ttf', fontdir=None, size=None):
    if fontdir is None:
        raise FontNotFoundError("Font '%s' not found." % name)
    fontpath = _getfontfile(name, f, fontdir)
    if fontpath is None:
        raise FontNotFoundError("Font '%s' not found." % name)
    if size is None:
        font = ImageFont.truetype(fontpath)
    else:
        font = ImageFont.truetype(fontpath, size=size)
    return font
